#!/usr/bin/lua

-- wbgen2, (c) 2010 Tomasz Wlostowski/CERN BE-Co-HT
-- LICENSED UNDER GPL v2

-- File: cgen_vhdl.lua
-- The Verilog code generator



VLOG_WIRE = 1;
VLOG_REG = 2;

function cgen_verilog_header()
    emit("//////////////////////////////////////////////////////////////////////////////////////");
    emit("// Title          : Wishbone slave core for "..periph.name);
    emit("//////////////////////////////////////////////////////////////////////////////////////");
    emit("// File           : "..options.output_hdl_file);
    emit("// Author         : auto-generated by wbgen2 from "..input_wb_file);
    emit("// Created        : "..os.date());
    emit("// Standard       : Verilog 2001");
    emit("//////////////////////////////////////////////////////////////////////////////////////");
    emit("// THIS FILE WAS GENERATED BY wbgen2 FROM SOURCE FILE "..input_wb_file);
    emit("// DO NOT HAND-EDIT UNLESS IT'S ABSOLUTELY NECESSARY!");
    emit("//////////////////////////////////////////////////////////////////////////////////////");
		emit("");
end

function cgen_verilog_module()
	local last;

	indent_zero();
	emit ("module "..periph.hdl_entity.." (");
	indent_right();

  for i=1,table.getn(g_portlist) do
		local port = g_portlist[i];
		  
		if(i == table.getn(g_portlist)) then
			last = true;
		else
		  last = false;
		end

		if(port.comment ~= nil) then
			emitx("// "..port.comment.."\n");
		end
	
	
		local dirstr;
		
		if(port.dir=="in") then
		 dirstr="input";
		elseif(port.dir=="out") then
		 dirstr="output";
		elseif(port.dir=="inout") then
		 dirst="inout";
		end

		print("Port: ", port.name, "type ", port.vlog_type);
	
		if(port.vlog_type == VLOG_REG) then
		 dirstr=dirstr.." reg";
		end

		local rangestr="";

		if(port.range > 0) then
		  rangestr = "["..(port.range-1)..":0]";
		end    

	
		local line = string.format("%-11s %-6s %s", dirstr, rangestr, port.name);

		      
		line=line..csel(last, "", ",");
		emit(line);
  end

    
	indent_left();
 	emit(");");
	indent_left();
	emit("");

	for i,v in pairs (g_siglist) do

		local rwire=csel(v.vlog_type==VLOG_REG, "reg", "wire");
		local rangestr="";

		if(v.range > 0) then
			rangestr=string.format("[%d:0] ", v.range-1);
		end

		emit(string.format("%-5s %-7s %s;", rwire, rangestr, v.name));
	end
	emit("");

	indent_right();
	
end

function cgen_verilog_ending()
	indent_left();
	emit("endmodule");
end




-- Generates the vlog code from syntax tree <tree>, writing it into previously chosen file.
function cgen_generate_verilog_code(tree)

local inside_process = false;

function find_code(node, t)
	for i,v in ipairs(node) do if ((v.t ~= nil) and (v.t == t)) then return v; end end
	return nil;
end

function cgen_verilog_syncprocess(node)
	local vrst = find_code(node.code, "reset");

  if(vrst ~= nil and options.reset_type=="asynchronous") then
 		emit("always @(posedge "..node.clk.." or "..csel(vrst.level==0, "negedge", "posedge").." rst_n_i) begin")
 	else
 		emit("always @(posedge "..node.clk..") begin");
 	end

	indent_right();

	inside_process = true;

	local vpe = find_code(node.code, "posedge");
			
	if(vpe == nil) then die("verilog code generation error: no vposedge defined for vsyncprocess"); end

	if(vrst ~= nil) then
		emit("if ("..node.rst.." == 1'b"..vrst.level..") begin ");
		indent_right();
		recurse(vrst.code);
		indent_left();
		emit("end else begin");
		indent_right();
	end

	recurse(vpe.code);		

  if(vrst ~= nil) then
		indent_left();
		emit("end");
 	end

  indent_left();
	emit("end");	
	emit("");

	inside_process = false;

end

function node_typesize(node)
	local ts = {};
	local sig;

	print(node);

	if(type(node) == "table") then
		print("table!");
		if(node.t ~= nil and node.t == "index") then
				sig = cgen_find_sigport(node.name);

				ts.h=node.h;
				ts.l=node.l;
				ts.sig=sig;
				ts.name=sig.name;
				ts.type=sig.type;
				ts.size = csel(ts.l==nil,1,ts.h-ts.l+1);
			
				return ts;			
	 	else
				print("expr2: ", node.t);
				ts.type = EXPRESSION;
				ts.code = node;
				return ts;
		end

	 elseif(type(node) == "string") then
		print("string-type node: "..node);
		sig = cgen_find_sigport(node);
		ts.sig=sig;
		ts.size = sig.range;
		ts.type = sig.type;
		ts.name = node;
		return ts;
	 elseif(type(node) == "number") then
		ts.type = INTEGER;
		ts.name = node;
		ts.size = 0;
		return ts;
	else 
		print("Unknown node?");
end
end

function gen_subrange(t)
	if(t.h ~= nil and t.l == nil) then
		return t.name.."["..t.h.."]";
	elseif(t.h ~= nil and t.l ~= nil) then
		return t.name.."["..t.h..":"..t.l.."]";
	else
		return t.name;
	end
end

function calc_size(t)
	if(t.h ~= nil and t.l == nil) then
		return 1;
	elseif(t.h ~= nil and t.l ~= nil) then
		return t.h-t.l+1; 
	else
		local sig= cgen_find_sigport(t.name);
		return sig.range;
	end
end


function cgen_verilog_assign(node)
	local tsd = node_typesize(node.dst);
	local tss = node_typesize(node.src);

--	for i,v in pairs(tsd) do
--	print(i,v);
--	end

-- vlog synchronous assignment
	if(inside_process) then
	
	if(tss.type == EXPRESSION) then
		print("EXPR!");
		emiti();
		emitx(gen_subrange(tsd).." <= ");
		tsd.sig.vlog_type = VLOG_REG;
		recurse({tss.code});
		emitx(";\n");
	else
		emit(gen_subrange(tsd).." <= "..gen_subrange(tss)..";");
		tsd.sig.vlog_type = VLOG_REG;
	end
	else -- vlog combinatorial assignment
	
		emit("assign "..gen_subrange(tsd).." = "..gen_subrange(tss)..";");
		tsd.sig.vlog_type = VLOG_WIRE;
	end
end

function cgen_verilog_if(node)
	emiti(); emitx("if (");
--	print("EMIT IF:", node.cond);

	for i,v in pairs(node.cond) do print(i,v); end
	recurse(node.cond);
	emitx(") begin\n");
	if(node.code_else ~= nil) then
	indent_right();
	recurse(node.code);
	indent_left();
	emit("end else begin");
	indent_right();
	recurse(node.code_else);
	indent_left();
	emit("end");
	else
	indent_right();
	recurse(node.code);
	indent_left();
	emit("end");

	end
	
end

function cgen_verilog_not(node)
	local tsa = node_typesize(node.a);

	emitx("! ");

	if(tsa.type == EXPRESSION) then
		emitx("("); recurse({node.a}); emitx(")");
	else
		emitx(gen_subrange(tsa));
	end
end


function cgen_verilog_op(node, op)
	local tsa = node_typesize(node.a);
	local tsb = node_typesize(node.b);

	if(tsa.type == EXPRESSION) then
		emitx("("); recurse({node.a}); emitx(")");
	else
		emitx(gen_subrange(tsa));
	end

	if(op=="eq") then emitx(" == "); end
	if(op=="and") then emitx(" && "); end
	if(op=="or") then emitx(" || "); end
	if(op=="sub") then emitx(" - "); end
	if(op=="add") then emitx(" + "); end

	if(tsb.type == EXPRESSION) then
		emitx("("); recurse({node.b}); emitx(")");
	else
		emitx(gen_subrange(tsb));
	end
end


function cgen_verilog_comment(node)
	emitx("// "..node.str.."\n");
end


-- switch/case statement --
function cgen_verilog_switch(node)

	local tsa = node_typesize(node.a);

	emiti(); emitx("case ");

	if(tsa.type == EXPRESSION) then
		emitx("("); recurse({node.a}); emitx(")");
	else
		emitx("("..gen_subrange(tsa)..")");
	end	

	emit("");

	for i,v in pairs(node.code)  do
		print("CA: ", v.t);

		if(v.t == "case") then			
			emit(string.format("%d'h%x: begin", tsa.size, v.a));
			indent_right();
			recurse({v.code});
			indent_left();
			emit("end");
		elseif(v.t == "casedefault") then			
			emit("default: begin");
			indent_right();
			recurse({v.code});
			indent_left();
			emit("end");
		end	


	end

	emit("endcase");

end



function recurse(t)


	print("recurse: ", t);

	for i,v in pairs(t) do
		print("-> i ",i,"v ",v, "vt ",v.t);

		if(v.t == nil) then
			recurse(v);
		elseif(v.t == "comment") then
			cgen_verilog_comment(v);
		elseif(v.t == "syncprocess") then
		 cgen_verilog_syncprocess(v);
		elseif(v.t=="assign") then
		 cgen_verilog_assign(v);		
		elseif(v.t=="if") then
			cgen_verilog_if(v);
		elseif(v.t=="eq" or v.t=="sub" or v.t=="add" or v.t == "or" or v.t=="and") then
			cgen_verilog_op(v,v.t);
		elseif(v.t=="switch") then
			cgen_verilog_switch(v);
		elseif(v.t=="not") then
			cgen_verilog_not(v);
		else
			print("unimpl: "..v.t);
			os.exit(-1);	
		end
	end
	print("end recurse");
end


cgen_new_snippet();
cgen_verilog_header();
local c_header = cgen_get_snippet();

cgen_new_snippet();
recurse(tree);
cgen_verilog_ending();
local c_body = cgen_get_snippet();

-- generate the module definions and signals - this must done after the module body has been generated in order to determine whether to declare things
-- as regs or wires.

cgen_new_snippet();
cgen_verilog_module();
local c_mod = cgen_get_snippet();

cgen_write_snippet(c_header);
cgen_write_snippet(c_mod);
cgen_write_snippet(c_body);

--cgen_write_snippet();
end
