
function gen_wishbone_ports()
    local ports = {
			port(BIT, 0, "in", "rst_n_i"),
			port(BIT, 0, "in", "wb_clk_i"),
		};
	
    if(address_bus_width > 0 ) then
			table_join(ports, { port(SLV, address_bus_width, "in", "wb_addr_i") });
    end

    table_join(ports, {
	port(SLV, DATA_BUS_WIDTH, "in", "wb_data_i"),
	port(SLV, DATA_BUS_WIDTH, "out", "wb_data_o"),
	port(BIT, 0, "in", "wb_cyc_i"),
	port(SLV, math.floor((DATA_BUS_WIDTH+7)/8), "in", "wb_sel_i"),
	port(BIT, 0, "in", "wb_stb_i"),
	port(BIT, 0, "in", "wb_we_i"),
	port(BIT, 0, "out", "wb_ack_o")
    });
    
	add_global_ports(ports);
end


function gen_wishbone_signals()

local wb_sigs = {  signal(BIT, 0, "wb_ack_regbank"),
								 	 signal(UNSIGNED, 4, "ack_cntr"),
									 signal(BIT,0, "ack_in_progress"),
									 signal(SLV, DATA_BUS_WIDTH, "rddata_reg"),
									 signal(SLV, DATA_BUS_WIDTH, "wrdata_reg"),
 									 signal(SLV, DATA_BUS_WIDTH/8	, "bwsel_reg"),
									 signal(SLV, address_bus_width, "rwaddr_reg"),
									 signal(BIT, 0, "bus_clock")
 };

	add_global_signals(wb_sigs);



end


-- generates the entire Wishbone bus access logic
function gen_bus_logic_wishbone()
  local s;
  local ramcount = 0;

-- count the RAMs in the design
  foreach_reg(function(reg) 
  							if(reg.__type == TYPE_RAM) then 
  								ramcount = ramcount + 1; 
  							end
  						end 
  						);

	print("ramcount: ", ramcount);

       
	periph.ramcount = ramcount;
    periph.fifocount = 0;

	gen_wishbone_ports();
	gen_wishbone_signals();

  foreach_reg(function(reg) 
			gen_abstract_code(reg);
    end );

	local resetcode={};
	local ackgencode={};
	local preackcode={};
	
  foreach_field(function(field, reg) 
    table_join(resetcode, field.reset_code_main); 
  end );

  foreach_reg(function(reg) 
    table_join(resetcode, reg.reset_code_main); 
  end );


  foreach_reg(function(reg) 
								if(reg.__type == TYPE_REG) then
	    						foreach_subfield(reg, function(field, reg) 
							     table_join(ackgencode, field.ackgen_code); 
	    						end );
								end
					    end);


  foreach_reg(function(reg) 
								if(reg.__type == TYPE_REG) then
	    						foreach_subfield(reg, function(field, reg) 
							     table_join(preackcode, field.ackgen_code_pre); 
	    						end );
								end
					    end);



	local fsmcode={};

  foreach_reg(function(reg) 
		if(reg.__type == TYPE_REG) then
	    local acklen = find_max(reg, "acklen");
			local rcode={};
			local wcode={};
			
			foreach_subfield(reg, function(field, reg) table_join(wcode, field.write_code); end );
			foreach_subfield(reg, function(field, reg) table_join(rcode, field.read_code); end );		

			local rwcode = {
				vif(vequal("wb_we_i" ,1), {
		   		wcode
				}, {
					rcode
				}); }; 

						
			table_join(rwcode, { va("wb_ack_regbank", csel((acklen==1),1,0)); } );
			table_join(rwcode, { va("ack_cntr", math.max(acklen-1, 0)); } );
			table_join(rwcode, { va("ack_in_progress", 1); } ); 

	   	if(regbank_address_bits > 0) then
	   		rwcode = { vcase(reg.base, rwcode); }; 
	   	end
			
			table_join(fsmcode, rwcode);

	  end
	end );




		 


  if(regbank_address_bits > 0) then

		table_join(fsmcode, { vcasedefault({
			va("ack_in_progress", 0);
			va("ack_cntr", 0);
			va("wb_ack_regbank", 0);
		}); });

   	fsmcode = { vswitch(vi("rwaddr_reg", regbank_address_bits - 1, 0), fsmcode); };
  end
  
  if(periph.ramcount > 0) then
	local ramswitchcode = { vcase (0, fsmcode); };
	
	

  foreach_reg(function(reg) 
  							if(reg.__type == TYPE_RAM) then 
  								table_join(ramswitchcode, { vcase(reg.select_bits , {
  												vif(vequal("wb_we_i" ,1), {
											   		va(reg.full_prefix.."_wr_int", 1);
											   		va("ack_cntr", 0);
											   		va("ack_in_progress", 1);
													}, {
														va(reg.full_prefix.."_rd_int", 1);
											   		va("ack_cntr", 1);
											   		va("ack_in_progress", 1);
													});
  								
  								})} );

									table_join(preackcode, {
							   		va(reg.full_prefix.."_wr_int", 0);
									});

									table_join(ackgencode, {
							   		va(reg.full_prefix.."_rd_int", 0);
									});
									
  							end
  						end 
  						);
	

	table_join(ramswitchcode, { vcasedefault({
												va("wb_ack_regbank",0);
												va("ack_in_progress",0);
												 }) });


  	fsmcode = { vswitch(vi("rwaddr_reg", address_bus_width-1, address_bus_width - address_bus_select_bits), ramswitchcode); };
  end


	fsmcode = { vif(vand(vequal("wb_cyc_i", 1), vequal("wb_stb_i", 1)), { fsmcode }, { va("wb_ack_regbank", 0) }); };	

	local code = {
	vsyncprocess("wb_clk_i", "rst_n_i", {
	 vreset(0, {
		va("wb_ack_regbank", 0);
		va("ack_in_progress", 0);
	  va("ack_cntr", 0);
	  va("rddata_reg", 0);
	  resetcode
	 });

	 vposedge ({
	   vif(vequal("ack_in_progress",1), {
			 vif(vequal("ack_cntr", 0), {
				 ackgencode;
					va("wb_ack_regbank", 0);
					va("ack_in_progress", 0);
			 }, {
				 preackcode;
				 va("ack_cntr", vsub("ack_cntr", 1));
				 vif(vequal("ack_cntr", 1), {
					 va("wb_ack_regbank", 1);
					} , {
					 va("wb_ack_regbank", 0);
 			 	});
		 });}, { -- else ack_in_progress = 0
			fsmcode			 
		 });
		});
	});

	va("wrdata_reg", "wb_data_i");
	va("rwaddr_reg", "wb_addr_i");	
	va("bwsel_reg", "wb_sel_i");	
	va("bus_clock", "wb_clk_i");

	};		 


	if(periph.ramcount > 0) then
		if(options.register_data_output = false) then
			local sens_list = {};
			local output_reg_process = { vcombprocess(
		
		
		
		
		end
	
	end


  foreach_reg(function(reg) 
								if(reg.extra_code ~= nil) then
									table_join(code, {vcomment("extra code for reg/fifo/mem: "..reg.name);});
									table_join(code, reg.extra_code);
								end

						   foreach_subfield(reg, function(field, reg) table_join(code, field.extra_code); end );
							end);
    
	return code;
end

