#!/usr/bin/lua

wbgen2_version="0.3"

wbgen2_libdir="/home/slayer/wbgen2_svn/wbgen2"

device_family="altera_cyclone3";

options = {};
options.reset_type = "asynchronous";
options.target_interconnect = "wb-classic";
options.register_data_output = false;

dofile(wbgen2_libdir.."/wbgen_common.lua");
dofile(wbgen2_libdir.."/cgen_common.lua");
dofile(wbgen2_libdir.."/cgen_vhdl.lua");
dofile(wbgen2_libdir.."/cgen_verilog.lua");
dofile(wbgen2_libdir.."/wbgen_regbank.lua");
dofile(wbgen2_libdir.."/wbgen_rams.lua");
dofile(wbgen2_libdir.."/target_wishbone.lua");

function chk_nil(p,s)
	if(p == nil) then
		die(s.." expected.");
	end
	return p;
end


function parse_args(arg)
	local n=1;

	if(arg[1] == nil) then
		print("wbgen2 version "..wbgen2_version);
		print("(c) Tomasz Wlostowski/CERN BE-Co-HT 2010");
		print("");
		print("usage: "..arg[0].." input_file.wb -t target -vo output.vhdl -co output.c -do output.html [additional options]");
		print("where target is the target FPGA architecture [altera/xilinx]");
		print("");
		print("Additional options: ");
		print("--gen-reg-constants              - generates VHDL constants containing addresses of all registers. Useful for writing testbenches.");
		print("--gen-vlog-constants file.v      - generates Verilog constants containing addresses of all registers and writes them to file.v. Useful for writing testbenches.");
		print("");
		os.exit(0);
	end
	
	input_wb_file = arg[1];

	vhdl_gen_reg_constants = false;
	vlog_gen_reg_constants = false;

	n=2;
	while(arg[n] ~= nil) do
		local sw = arg[n];
		
		if(sw == "-vo") then
			options.output_hdl_file = chk_nil(arg[n+1], "HDL output filename expected");
			n=n+2;
		elseif(sw == "-co") then
			output_c_file = chk_nil(arg[n+1], "C header output filename expected");
			n=n+2;
		elseif(sw == "--gen-reg-constants") then
			vhdl_gen_reg_constants = true;
			n=n+1;
		elseif(sw == "--gen-vlog-constants") then
			output_vlog_constants_file = chk_nil(arg[n+1],"Verilog constants filename expected");
			vlog_gen_reg_constants = true;
			n=n+2;
		else
			n=n+1;
		end
	
	end

end

parse_args(arg);

dofile(input_wb_file);

if(periph == nil) then die ("missing peripheral declaration"); end
    
foreach_reg(
    function(reg) 
	reg.total_size=0; 
	reg.current_offset=0;
    end
);

foreach_field( fix_prefix );
foreach_field( fix_access );

foreach_reg( fix_prefix );

periph = fix_prefix(periph);

foreach_field(calc_size);
foreach_reg(check_max_size);
foreach_field(calc_field_offsets);

foreach_reg(calc_address_sizes);
assign_addresses();

tree=gen_bus_logic_wishbone();

cgen_build_signals_ports();
cgen_generate_hdl_init(options.output_hdl_file)
cgen_generate_vhdl_code(tree);
cgen_generate_hdl_done();

