#!/usr/bin/lua


function gen_code_ram(ram)
  local prefix = string.lower(periph.hdl_prefix.."_"..ram.hdl_prefix);
	local maps = {};
	
	print("GENRAMCODE!"..ram.name);

-- generate the RAM-related ports
	ram.full_prefix = prefix;
	ram.signals = {};
	
  ram.ports = { port (SLV, ram.addr_bits - ram.wrap_bits, "in", prefix.."_addr_i", "Ports for RAM: "..ram.name ) };
	ram.reset_code_main = {};

	table_join(maps,			{ vpm ("clk_a_i", "bus_clock");
													vpm ("clk_b_i", csel(ram.clock ~= nil, ram.clock, "bus_clock"));
													vpm ("addr_b_i", prefix.."_addr_i"); });


  if(match(ram.access_dev, {READ_ONLY, READ_WRITE})) then
		
		table_join(ram.ports, 	{	port(SLV, ram.width, "out", prefix.."_data_o", "Read data output"),
															port(BIT, 0, "in", prefix.."_rd_i", "Read strobe input (active high)") });


		table_join(maps,				{ vpm ("data_b_o", prefix.."_data_o");
															vpm ("rd_b_i", prefix.."_rd_i"); } );
																		
  end

  if(match(ram.access_dev, {WRITE_ONLY, READ_WRITE})) then
		table_join(ram.ports, { port(SLV, ram.width, "in", prefix.."_data_i", "Write data input"), 
														port(BIT, 0, "in", prefix.."_wr_i", "Write strobe (active high)") });

		table_join(maps, 			{ vpm ("data_b_i", prefix.."_data_i");
														vpm ("wr_b_i", prefix.."_wr_i"); });

		if(ram.byte_select == true and ram.width >= 16) then
			table_join(ram.ports, { port (SLV, ram.width/8, "in", prefix.."_bwsel_i", "Byte select input (active high)") } );
			table_join(maps, 			{ vpm ("bwsel_b_i", prefix.."_bwsel_i"); } );
		end	
  end


	table_join(maps,					{ vpm ("addr_a_i", vi("rwaddr_reg", log2up(ram.size)-1, 0)) });

  if(match(ram.access_bus, {READ_ONLY, READ_WRITE})) then
		
		table_join(ram.signals, { signal(SLV, ram.width,  prefix.."_rddata_int"),
															signal(BIT, 0, prefix.."_rd_int") } );


		table_join(maps,				{ vpm ("data_a_o", vi(prefix.."_rddata_int", ram.width-1, 0));
															vpm ("rd_a_i", prefix.."_rd_int"); } );

		table_join(ram.reset_code_main, { va(prefix.."_rd_int", 0); });
																		
  end

  if(match(ram.access_bus, {WRITE_ONLY, READ_WRITE})) then


		table_join(ram.signals, { signal(BIT, 0, prefix.."_wr_int") } );

		table_join(maps, 			{ vpm ("data_a_i", vi("wrdata_reg", ram.width-1, 0));
														vpm ("wr_a_i", prefix.."_wr_int"); });

		table_join(ram.reset_code_main, { va(prefix.."_wr_int", 0); });

		if(ram.byte_select == true and ram.width >= 16) then
			table_join(maps, 			{ vpm ("bwsel_a_i", vi("bwsel_reg", math.floor(ram.width/8)-1, 0)); } );
		end	
  end



-- fill in all the generic mappings
	table_join(maps, 					{ vgm ("g_data_width", ram.width);
															vgm ("g_size", ram.size);
															vgm ("g_addr_width", log2up(ram.size));
															vgm ("g_dual_clock", csel(ram.clock ~= nil, "true", "false"));
															vgm ("g_use_bwsel", csel(ram.byte_select == true, "true", "false"));
															});

	ram.extra_code = { 			vcomment ("RAM block instantiation for memory: "..ram.name);
													vinstance (prefix.."_raminst", "wbgen2_dpssram", maps ); };

end
